@@ -3,6 +3,7 @@ import logging
 
 from ConfigParser import ConfigParser
 from ConfigParser import NoOptionError
+from twisted.python import log
 from buildbot.buildslave import BuildSlave
 from buildbot.config import BuilderConfig
 
@@ -12,12 +13,14 @@ from steps import PgSetProperties
 from buildbot.steps.shell import ShellCommand
 from buildbot.steps.transfer import FileDownload
 from buildbot.process.properties import WithProperties
-from buildbot.process.properties import Property
 from buildbot.schedulers.basic import SingleBranchScheduler
 
 from . import capability
-from scheduler import MirrorChangeFilter
-from utils import comma_list_sanitize
+from . import watch
+from . import subfactories
+from . import buildouts
+
+from .utils import BUILD_UTILS_PATH
 from version import Version
 from version import VersionFilter
 
@@ -28,8 +31,6 @@ BUILDSLAVE_KWARGS = {  # name -> validating callable
 
 BUILDSLAVE_REQUIRED = ('password',)
 
-BUILD_UTILS_PATH = os.path.join(os.path.split(__file__)[0], 'build_utils')
-
 logger = logging.getLogger(__name__)
 
 # Running buildouts in parallel on one slave fails
@@ -40,17 +41,10 @@ buildout_lock = locks.SlaveLock("buildout")
 class BuildoutsConfigurator(object):
     """Populate buildmaster configs from buildouts and external slaves.cfg.
 
-    There are three dictionnaries of methods:
-       buildout_dl_steps : name -> method returning the main buildout config
-                           file name and a list of steps to
-                           construct the buildout configuration slave-side.
-       post_dl_steps: name-> method returning the main buildout config file
-                             name and a list of steps to be inserted
-                             between the buildout conf retrieval and the main
-                             buildout run.
-       post_buildout_steps : name -> method returning a list of steps to be
-                             added after buildout has run and testing db is
-                             done
+    Use the three subfactories of steps from ``subfactories``:
+       - buildout_download (see ``subfactories.download``)
+       - post_download (see ``subfactories.postdownload``)
+       - post_buildout (see ``subfactories.postbuildout``)
     """
 
     cap2environ = dict(
@@ -63,8 +57,13 @@ class BuildoutsConfigurator(object):
                                  },
                         ))
 
+    vcs_master_url_rewrite_rules = ()
+
+    tree_stable_timer = 600
+
     def __init__(self, buildmaster_dir,
                  manifest_paths=('buildouts/MANIFEST.cfg',),
+                 slaves_path='slaves.cfg',
                  capability_options_to_environ=None):
         """Attach to buildmaster in which master_cfg_file path sits.
         """
@@ -72,6 +71,7 @@ class BuildoutsConfigurator(object):
         self.build_factories = {}  # build factories by name
         self.factories_to_builders = {}  # factory name -> builders playing it
         self.manifest_paths = manifest_paths
+        self.slaves_path = slaves_path
         if capability_options_to_environ is not None:
             self.cap2environ = capability_options_to_environ
 
@@ -81,10 +81,13 @@ class BuildoutsConfigurator(object):
         self.cap2environ[capability_name] = options2environ
 
     def populate(self, config):
-        config.setdefault('slaves', []).extend(self.make_slaves('slaves.cfg'))
+        config.setdefault('slaves', []).extend(
+            self.make_slaves(self.slaves_path))
         map(self.register_build_factories, self.manifest_paths)
         config.setdefault('builders', []).extend(
             self.make_builders(master_config=config))
+        self.init_watch()
+        config.setdefault('change_source', []).extend(self.make_pollers())
         config.setdefault('schedulers', []).extend(self.make_schedulers())
 
     def path_from_buildmaster(self, path):
@@ -94,6 +97,18 @@ class BuildoutsConfigurator(object):
 
         return os.path.join(self.buildmaster_dir, path)
 
+    def init_watch(self):
+        self.watcher = watch.MultiWatcher(
+            self.manifest_paths,
+            url_rewrite_rules=self.vcs_master_url_rewrite_rules)
+        self.watcher.read_branches()
+
+    def make_pollers(self):
+        """Return pollers for watched repositories.
+        """
+        # lp resolution can lead to dupes
+        return list(set(self.watcher.make_pollers()))
+
     def make_slaves(self, conf_path='slaves.cfg'):
         """Create the slave objects from the file at conf_path.
 
@@ -140,83 +155,63 @@ class BuildoutsConfigurator(object):
 
         return slaves
 
-    def buildout_standalone_dl_steps(self, cfg_tokens, manifest_dir):
-        """Return slave side path and steps about the buildout.
+    def steps_bootstrap(self, buildout_slave_path, options, eggs_cache,
+                        **step_kw):
+        """return a list of steps for buildout bootstrap.
 
-        The first returned value is the expected path from build directory
-        The second is an iterable of steps to get the buildout config file
-        and the related needed files (extended cfgs, bootstrap.py).
-
-        manifest_dir is the path (interpreted from buildmaster dir) to the
-        directory in with the manifest file sits.
-        """
-        if len(cfg_tokens) != 1:
-            raise ValueError(
-                "Wrong standalong buildout specification: %r" % cfg_tokens)
-
-        conf_path = cfg_tokens[0]
-        conf_name = os.path.split(conf_path)[-1]
-        conf_path = os.path.join(manifest_dir, conf_path)
-        bootstrap_path = os.path.join(manifest_dir, 'bootstrap.py')
-        return conf_name, (FileDownload(mastersrc=bootstrap_path,
-                                        slavedest='bootstrap.py'),
-                           FileDownload(mastersrc=conf_path,
-                                        slavedest=conf_name),
-                           )
-
-    def buildout_hg_dl_steps(self, cfg_tokens, manifest_dir):
-        """Return slave side path and steps about the buildout.
-
-        The first returned value is the expected path from build directory
-        The second is an iterable of steps to get the buildout config file
-        and the related needed files (extended cfgs, bootstrap.py).
-
-        manifest_dir is not used in this downloader.
-        """
-        if len(cfg_tokens) != 3:
-            raise ValueError(
-                "Wrong standalong buildout specification: %r" % cfg_tokens)
-
-        url, branch, conf_path = cfg_tokens
-        return conf_path, (
-            FileDownload(
-                mastersrc=os.path.join(BUILD_UTILS_PATH, 'buildout_hg_dl.py'),
-                slavedest='buildout_hg_dl.py',
-                haltOnFailure=True),
-            ShellCommand(
-                command=['python', 'buildout_hg_dl.py', url, branch],
-                description=("Retrieve buildout", "from hg",),
-                haltOnFailure=True,
-            )
-        )
-
-    def buildout_hg_tag_dl_steps(self, cfg_tokens, manifest_dir):
-        """Steps to retrieve the buildout dir as a Mercurial tag.
-
-        Useful for release/packaging oriented builds.
-        The tag name is read from build properties.
-        The clone is made outside of the main build/ directory, that must
-        stay pristine to test the produced packages.
+        step_kw will be passed to the step constructor. Known use-case:
+        change workdir in packaging step.
+        options prefixed with 'bootstrap-' are applied
         """
 
-        if len(cfg_tokens) != 2:
+        bootstrap_prefix = 'bootstrap-'
+        bootstrap_options = dict((k[len(bootstrap_prefix):], v.strip())
+                                 for k, v in options.items()
+                                 if k.startswith(bootstrap_prefix))
+        bootstrap_options.setdefault('version', '2.1.1')
+        forbidden = set(('eggs', 'help', 'find-links'))
+        if not forbidden.isdisjoint(bootstrap_options):
             raise ValueError(
-                "Wrong hgtag buildout specification: %r" % cfg_tokens)
+                "The following bootstrap options are forbidden: %r" % list(
+                    forbidden))
 
-        url, conf_path = cfg_tokens
-        tag = Property('buildout-tag')
-        return conf_path, (
-            FileDownload(
-                mastersrc=os.path.join(BUILD_UTILS_PATH, 'buildout_hg_dl.py'),
-                slavedest='../src/buildout_hg_dl.py',
-                haltOnFailure=True),
-            ShellCommand(
-                command=['python', 'buildout_hg_dl.py', '-t', 'tag', url, tag],
-                workdir='./src',
-                description=("Retrieve buildout", "tag", tag, "from hg",),
-                haltOnFailure=True,
-            )
-        )
+        known_bootstraps = ('v1', 'v2')
+
+        bootstrap_type = bootstrap_options.pop('type', 'v1').lower()
+        if not bootstrap_type in known_bootstraps:
+            raise ValueError(
+                "Unknown bootstrap type: %r. Known ones are %r" % (
+                    bootstrap_type, known_bootstraps))
+
+        find_links_opt = dict(v1='--eggs', v2='--find-links')[bootstrap_type]
+
+        venv_opt = bootstrap_options.pop('virtualenv', 'false').strip()
+        if venv_opt.lower() == 'true':
+            env = dict(PATH=["${HOME}/openerp-env/bin", "${PATH}"])
+        else:
+            env = None
+
+        # bootstrap script path is by default relative to the
+        # build directory, not the buildout config. Indeed, buildouts
+        # bootstrapped from another directory need extra care to work out
+        # of the box, and their needs depend on the exact situation. So
+        # we expect in that case users to issue a proper relative
+        # path in that situation, that is likely to be the simplest of their
+        # tunings
+        command = ['python', bootstrap_options.get('script', 'bootstrap.py'),
+                   find_links_opt, eggs_cache,
+                   '-c', buildout_slave_path]
+        command.extend('--%s=%s' % (k, v)
+                       for k, v in bootstrap_options.items())
+
+        return [ShellCommand(command=command,
+                             name='bootstrap',
+                             description="bootstrapping",
+                             descriptionDone="bootstrapped",
+                             haltOnFailure=True,
+                             env=env,
+                             **step_kw)
+                ]
 
     def make_factory(self, name, buildout_slave_path, buildout_dl_steps,
                      options):
@@ -253,19 +248,16 @@ class BuildoutsConfigurator(object):
                                      warnOnFailure=False,
                                      hideStepIf=True,
                                      ))
-
-        for dl_step in buildout_dl_steps:
-            factory.addStep(dl_step)
+        map(factory.addStep, buildout_dl_steps)
 
         capability_env = capability.set_properties_make_environ(
             self.cap2environ, factory)
 
-        for line in options.get('post-dl-steps', 'standard').split(os.linesep):
-            post_dl_steps = self.post_dl_steps[line]
-            buildout_slave_path, steps = post_dl_steps(
+        for line in options.get('post-dl-steps', 'noop').split(os.linesep):
+            subfactory = subfactories.post_download[line]
+            buildout_slave_path, steps = subfactory(
                 self, options, buildout_slave_path, environ=capability_env)
-            for step in steps:
-                factory.addStep(step)
+            map(factory.addStep, steps)
 
         factory.addStep(FileDownload(
             mastersrc=os.path.join(
@@ -286,19 +278,19 @@ class BuildoutsConfigurator(object):
                                      name="cachedirs",
                                      description="prepare cache dirs"))
 
-        factory.addStep(ShellCommand(command=['python', 'bootstrap.py',
-                                              '-c', buildout_slave_path],
-                                     haltOnFailure=True,
-                                     ))
+        map(factory.addStep,
+            self.steps_bootstrap(buildout_slave_path, options, eggs_cache))
 
         factory.addStep(ShellCommand(command=[
             'bin/buildout',
             '-c', buildout_slave_path,
             'buildout:eggs-directory=' + eggs_cache,
             'buildout:openerp-downloads-directory=' + openerp_cache,
-            'openerp:with_devtools=True',
-            'openerp:vcs-clear-locks=True',
-            'openerp:vcs-clear-retry=True',
+            'openerp:with_devtools=true',
+            'openerp:vcs-clear-locks=true',
+            'openerp:vcs-clear-retry=true',
+            'openerp:clean=true',
+            'openerp:vcs-revert=on-merge',
             WithProperties(
                 'openerp:options.db_port=%(cap_postgresql_port:-5432)s'),
             WithProperties(
@@ -308,6 +300,7 @@ class BuildoutsConfigurator(object):
             WithProperties(
                 'openerp:options.db_password='
                 '%(cap_postgresql_passwd:-False)s'),
+            WithProperties('openerp:options.db_name=%(testing_db)s')
         ],
             name="buildout",
             description="buildout",
@@ -317,95 +310,29 @@ class BuildoutsConfigurator(object):
             env=capability_env,
         ))
 
-        factory.addStep(ShellCommand(command=[
-            'psql', 'postgres', '-c',
-            WithProperties('DROP DATABASE IF EXISTS "%(testing_db)s"'),
-        ],
-            name='dropdb',
-            description=["dropdb", Property('testing_db')],
-            env=capability_env,
-            haltOnFailure=True,
-        ))
-
-        factory.addStep(ShellCommand(command=[
-            'psql', 'postgres', '-c',
-            WithProperties('CREATE DATABASE "%%(testing_db)s" '
-                           'TEMPLATE "%s"' % options.get('db_template',
-                                                         'template1')),
-        ],
-            name='createdb',
-            description=["createdb", Property('testing_db')],
-            env=capability_env,
-            haltOnFailure=True,
-        ))
+        for line in options.get('db-steps',
+                                'simple_create').split(os.linesep):
+            map(factory.addStep,
+                subfactories.db_handling[line.strip()](
+                    self, options, environ=capability_env))
 
         for line in options.get('post-buildout-steps',
                                 'standard').split(os.linesep):
-            post_buildout_steps = self.post_buildout_steps[line]
+            map(factory.addStep,
+                subfactories.post_buildout[line](
+                    self, options, buildout_slave_path,
+                    environ=capability_env))
 
-            for step in post_buildout_steps(self, options, buildout_slave_path,
-                                            environ=capability_env):
-                factory.addStep(step)
-
-        build_category = options.get('build-category')
-        if build_category:
-            factory.build_category = build_category.strip()
+        factory.options = options
         return factory
 
-    def post_dl_steps_standard(self, options, buildout_slave_path, environ=()):
-        return buildout_slave_path, ()
-
-    post_dl_steps = dict(standard=post_dl_steps_standard)
-
-    def post_buildout_steps_standard(self, options, buildout_slave_path,
-                                     environ=()):
-
-        environ = dict(environ)
-
-        steps = []
-
-        steps.append(ShellCommand(command=['rm', '-f', 'test.log'],
-                                  name="Log cleanup",
-                                  descriptionDone=['Cleaned', 'logs'],
-                                  ))
-
-        steps.append(ShellCommand(command=[
-            'bin/test_openerp', '-i',
-            comma_list_sanitize(options.get('openerp-addons', 'all')),
-            '-d', Property('testing_db'),
-            # openerp --logfile does not work with relative paths !
-            WithProperties('--logfile=%(workdir)s/build/test.log')],
-            name='testing',
-            description='testing',
-            descriptionDone='tests',
-            logfiles=dict(test='test.log'),
-            haltOnFailure=True,
-            env=environ,
-        ))
-
-        steps.append(ShellCommand(
-            command=["python", "analyze_oerp_tests.py", "test.log"],
-            name='analyze',
-            description="analyze",
-        ))
-
-        return steps
-
-    post_buildout_steps = dict(standard=post_buildout_steps_standard)
-
     def register_build_factories(self, manifest_path):
         """Register a build factory per buildout from file at manifest_path.
 
         manifest_path is interpreted relative to the buildmaster dir.
-        For now, only *standalone* buildouts are taken into account, meaning
-        that they are entirely described in one cfg file.
-
-        For easy inclusion of project-specific layouts, we might in the future
-        introduce directory buildouts and even VCS buildouts. For now,
-        developers have to contribute a single file meant for the buildbot.
         """
-        parser = ConfigParser()
-        parser.read(self.path_from_buildmaster(manifest_path))
+        parser = buildouts.parse_manifest(
+            self.path_from_buildmaster(manifest_path))
         manifest_dir = os.path.split(manifest_path)[0]
         registry = self.build_factories
 
@@ -417,7 +344,7 @@ class BuildoutsConfigurator(object):
                 continue
 
             btype = buildout[0]
-            buildout_downloader = self.buildout_dl_steps.get(btype)
+            buildout_downloader = subfactories.buildout_download.get(btype)
             if buildout_downloader is None:
                 raise ValueError("Buildout type %r in %r not supported" % (
                     btype, name))
@@ -428,10 +355,6 @@ class BuildoutsConfigurator(object):
                 name, conf_slave_path, dl_steps, dict(parser.items(name)))
             factory.manifest_path = manifest_path  # change filter will need it
 
-    buildout_dl_steps = dict(standalone=buildout_standalone_dl_steps,
-                             hgtag=buildout_hg_tag_dl_steps,
-                             hg=buildout_hg_dl_steps)
-
     def make_builders(self, master_config=None):
         """Spawn builders from build factories.
 
@@ -488,7 +411,7 @@ class BuildoutsConfigurator(object):
                     name='%s-postgresql-%s' % (factory_name,
                                                pg_version),
                     properties=dict(pg_version=pg_version),
-                    category=getattr(factory, 'build_category', None),
+                    category=factory.options.get('build-category', '').strip(),
                     factory=factory, slavenames=slavenames)
                 for pg_version, slavenames in meet_requires.items()
                 if pgvf is None or pgvf.match(Version.parse(pg_version))
@@ -513,16 +436,26 @@ class BuildoutsConfigurator(object):
         TODO at some point check if a big dedicated, single schedulers would
         not be preferable for buildmaster performance.
         """
-        fact_to_builders = self.factories_to_builders
 
-        def make_filter(factory_name):
-            """Make a Mirror Change Filter for factory with given name."""
-            return MirrorChangeFilter(
-                self.factory_to_manifest(factory_name, absolute=True),
-                factory_name)
+        schedulers = []
+        for factory_name, builders in self.factories_to_builders.items():
+            options = self.build_factories[factory_name].options
 
-        return [SingleBranchScheduler(name=factory_name,
-                                      change_filter=make_filter(factory_name),
-                                      treeStableTimer=60,
-                                      builderNames=builders)
-                for factory_name, builders in fact_to_builders.items()]
+            tree_stable_timer = options.get('tree-stable-timer')
+            if tree_stable_timer is not None:
+                tree_stable_timer = int(tree_stable_timer.strip())
+            else:
+                tree_stable_timer = self.tree_stable_timer
+
+            change_filter = self.watcher.change_filter(factory_name)
+            if change_filter is None:
+                continue
+            schedulers.append(SingleBranchScheduler(
+                name=factory_name,
+                change_filter=change_filter,
+                treeStableTimer=tree_stable_timer,
+                builderNames=builders))
+            log.msg("Scheduler %r is for builders %r "
+                    "with %r" % (factory_name, builders, change_filter))
+
+        return schedulers
