@@ -2,19 +2,12 @@ import os
 import hashlib
 import subprocess
 
+BUILD_UTILS_PATH = os.path.join(os.path.split(__file__)[0], 'build_utils')
+
+
 # can be overridden from command line tools such as update-mirrors,
 # for the version that has the buildbot hooks.
-vcs_binaries = dict(bzr='bzr', hg='hg')
-
-
-def mkdirp(path):
-    """Python equivalent for mkdir -p"""
-    if os.path.isdir(path):
-        return
-    parent, name = os.path.split(path)
-    if not os.path.isdir(parent):
-        mkdirp(parent)
-    os.mkdir(path)
+vcs_binaries = dict(bzr='bzr', hg='hg', git='git')
 
 
 def ez_hash(url):
@@ -75,9 +68,37 @@ def hg_pull(path, source, specs):
             subprocess.call(base_cmd + ('-b', spec[0]))
 
 
+def git_init_clone(path, source, specs):
+    """clone git repository and checkout branch."""
+    cmd = [vcs_binaries['git'], 'clone', source, path]
+    for spec in specs:
+        if len(spec) != 1:
+            raise ValueError("Invalid branch specification %r in "
+                             "git repo at %r", spec, source)
+        cmd.append('-b')
+        cmd.append(spec[0])
+    subprocess.call(cmd)
+
+
+def git_pull(path, source, specs):
+    """Pull from source to clone at path the specified branches."""
+    cmd = [vcs_binaries['git'], 'pull', source]
+    for spec in specs:
+        if len(spec) != 1:
+            raise ValueError("Invalid branch specification %r in "
+                             "git repo at %r", spec, source)
+        cmd.append(spec[0])
+    subprocess.call(cmd, cwd=path)
+
+
 def comma_list_sanitize(comma_list):
     """Sanitize a list given as a comma separated string.
 
     Will remove all internal whitespace
     """
     return ','.join([i.strip() for i in comma_list.split(',')])
+
+
+def bool_opt(options, name):
+    """Parse a boolean from a config section dict."""
+    return options.get(name, '').strip().lower() == 'true'
